//+------------------------------------------------------------------+
//|                                                     OBV DIVS.mq5 |
//|                                  Copyright 2021, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+

#property copyright "LOVE JOY PEACE PATIENCE KINDNESS GOODNESS FAITHFULNESS GENTLENESS SELF-CONTROL"
#property link      "https://www.therationalinvestor.com"
#property version   "1.00"
#property description "TRI's OBV w/ DIVS"

#include <MovingAverages.mqh>

// Notes: Taking obv w/ ema script and using default 5 bar fractal scripts to build this v1.0 div script out. 
// Add buffers and variables
// Add properties for indicator buffers
// Add setindexbuffer and plot index info
// Review IF statements in OnCalc and try to use existing or possibly use Ghost Willy approach to make multiple counters

//--- indicator settings
#property indicator_separate_window

#property indicator_buffers 4 // Was 2, update to 4
#property indicator_plots   4 // Was 2, update to 4

// OBV and EMA
#property indicator_type1     DRAW_LINE
#property indicator_color1    clrDarkTurquoise
#property indicator_label1    "OBV"
#property indicator_style2    STYLE_SOLID
#property indicator_width1    2

#property indicator_type2     DRAW_LINE                                                   
#property indicator_color2    clrCoral 
#property indicator_label2    "EMA on OBV"                               
#property indicator_style2    STYLE_SOLID                           
#property indicator_width2    2    

// Fractals
#property indicator_type3   DRAW_ARROW
#property indicator_type4   DRAW_ARROW
#property indicator_color3  Gray
#property indicator_color4  Gray
#property indicator_label3  "Fractal Up"
#property indicator_label4  "Fractal Down"                          

//--- input parametrs for OBV
input ENUM_APPLIED_VOLUME     InpVolumeType=VOLUME_TICK;       // Volumes

// Inputs for EMA on OBV
input int                     InpMAPeriod      = 13;           // EMA Period
input ENUM_MA_METHOD          InpMAMode        = MODE_EMA;     // MA Mode

//--- indicator buffer(s)
double ExtOBVBuffer[];  // OBV
double EMABuffer[];     // EMA on OBV

double ExtUpperBuffer[]; // Up Fractal
double ExtLowerBuffer[]; // Down Fractal

//--- 10 pixels upper from high price
int    ExtArrowShift=-10;

//+------------------------------------------------------------------+
//| On Balance vol initialization function                        |
//+------------------------------------------------------------------+

int OnInit()
  {
  
//---- name for DataWindow and indicator subwindow label
   IndicatorSetString(INDICATOR_SHORTNAME, "OBV DIVS" + "(" + string(InpMAPeriod) + ")");
   
//--- define indicator buffer
   SetIndexBuffer(0, ExtOBVBuffer, INDICATOR_DATA); // OBV Buffer and Index   
   SetIndexBuffer(1, EMABuffer, INDICATOR_DATA); // EMA on OBV Buffer and Index
   
   SetIndexBuffer(2,ExtUpperBuffer,INDICATOR_DATA); // Up Fractal
   SetIndexBuffer(3,ExtLowerBuffer,INDICATOR_DATA); // Down Fractal
   
   PlotIndexSetInteger(1, PLOT_DRAW_BEGIN, InpMAPeriod); // Waiting to plot until enough candles, 13 period by default. 
   
//--- set indicator digits
   IndicatorSetInteger(INDICATOR_DIGITS,2); // Changing to 2 from 0 for the EMA on OBV addition. 

// Taken from Fractals. // 
//--- sets first bar from what index will be drawn 
   PlotIndexSetInteger(2,PLOT_ARROW,217);
   PlotIndexSetInteger(3,PLOT_ARROW,218);
//--- arrow shifts when drawing
   PlotIndexSetInteger(2,PLOT_ARROW_SHIFT,ExtArrowShift);
   PlotIndexSetInteger(3,PLOT_ARROW_SHIFT,-ExtArrowShift);
//--- sets drawing line empty value--
   PlotIndexSetDouble(2,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   PlotIndexSetDouble(3,PLOT_EMPTY_VALUE,EMPTY_VALUE);
   
   return(INIT_SUCCEEDED);
   
  }
  
//+------------------------------------------------------------------+
//| Custom Indicator Deinitialization Function                       |
//+------------------------------------------------------------------+
/*void OnDeinit(const int reason)
{
   ObjectsDeleteAll(0, prefix, 1, OBJ_ARROW_UP);
   ObjectsDeleteAll(0, prefix, 1, OBJ_ARROW_DOWN);
   ChartRedraw(1);
}
*/  

//+------------------------------------------------------------------+
//| Calculate OBV by volume argument                                 |
//+------------------------------------------------------------------+

// Note: In Willy script this was moved up in the code. Triple check this. 

void CalculateOBV(int start_pos,
                  const int rates_total,
                  const int prev_calculated,
                  const double& close[],
                  const long& volume[])
  {
   for(int i=start_pos; i<rates_total-prev_calculated && !IsStopped(); i++)
     {
      double vol=(double)volume[i];    
      double prev_close=close[i-1];
      double curr_close=close[i];
            
      //--- fill ExtOBVBuffer
      if(curr_close<prev_close)
         ExtOBVBuffer[i]=ExtOBVBuffer[i-1]-vol;
      else
        {
         if(curr_close>prev_close)
            ExtOBVBuffer[i]=ExtOBVBuffer[i-1]+vol;
         else
            ExtOBVBuffer[i]=ExtOBVBuffer[i-1];
        }
        
        Print("i: " + i +" ExtOBVBuffer value: " + ExtOBVBuffer[i] + " prev_calc: " + prev_calculated + " vol: " + vol);
     }
  }
 
//+------------------------------------------------------------------+
//| On Balance vol                                                |
//+------------------------------------------------------------------+

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
        
   if(rates_total<5) // Could change this to be EMA input. Leaving 2 for now. // Fractals wants 5. Changing from 2 to 5. 
      return(0);

   if (rates_total==prev_calculated)   return (rates_total); 
       
   int start; // Variable from Fractals
   
//--- clean up arrays and sets start variable // Taken From Fractals Code
   if(prev_calculated<7)
     {
      start=2;
      ArrayInitialize(ExtUpperBuffer,EMPTY_VALUE);
      ArrayInitialize(ExtLowerBuffer,EMPTY_VALUE);
     }
   else
      start=rates_total-5;

//--- starting calculation
   int pos=prev_calculated-1;
   
//--- correct position, when it's first iteration
   if(pos<1)
     {
      pos=1;
      if(InpVolumeType==VOLUME_TICK)
         ExtOBVBuffer[0]=(double)tick_volume[0];
      else
         ExtOBVBuffer[0]=(double)volume[0];
     }

//--- main cycle
   if(InpVolumeType==VOLUME_TICK)
      CalculateOBV(pos,rates_total,prev_calculated, close,tick_volume);
   else
      CalculateOBV(pos,rates_total, prev_calculated, close,volume);
      
   //--- calculate the first moving MA for this indicator (OBV)
   ExponentialMAOnBuffer(rates_total, prev_calculated,
                      1,               // index, starting from which data for smoothing are available 
                      InpMAPeriod,     // period of the exponential average
                      ExtOBVBuffer,    // buffer to calculate average
                      EMABuffer);      // into this buffer locate value of the average

// Dropping in Fractals IF statements here //

//--- main cycle of calculations // Changing counter from i to j
//   for(int j=start; j<rates_total-3 && !IsStopped(); j++)
     for(int j=start; j<(rates_total-prev_calculated)-3 && !IsStopped(); j++)
     {
      //--- Upper Fractal
      if(ExtOBVBuffer[j]>ExtOBVBuffer[j+1] && ExtOBVBuffer[j]>ExtOBVBuffer[j+2] && ExtOBVBuffer[j]>=ExtOBVBuffer[j-1] && ExtOBVBuffer[j]>=ExtOBVBuffer[j-2]) // For OBV Div swap high[i] with ExtOBVBuffer[i]
         ExtUpperBuffer[j]=ExtOBVBuffer[j];
      else
         ExtUpperBuffer[j]=EMPTY_VALUE;

      //--- Lower Fractal
      if(ExtOBVBuffer[j]<ExtOBVBuffer[j+1] && ExtOBVBuffer[j]<ExtOBVBuffer[j+2] && ExtOBVBuffer[j]<=ExtOBVBuffer[j-1] && ExtOBVBuffer[j]<=ExtOBVBuffer[j-2])
         ExtLowerBuffer[j]=ExtOBVBuffer[j];
      else
         ExtLowerBuffer[j]=EMPTY_VALUE;
     
     Print(" j: " + j + " ExtOBVBuffer value: " + ExtOBVBuffer[j] + " prev_calc: " + prev_calculated + " rates_total: " + rates_total);
     Print(" ExtUpperBuffer value: " + ExtUpperBuffer[j] + " ExtLowerBuffer value: " + ExtLowerBuffer[j]);
     }
   
   Print("********* I AM DONE *************");
 
//--- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }


//+------------------------------------------------------------------+
